@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 2011-2021 The Octave Project Developers
+// Copyright (C) 2011-2022 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -53,6 +53,7 @@ namespace octave
     event_manager& evmgr = interp.get_event_manager ();
 
     evmgr.connect_link (m_octave_qobj.get_qt_interpreter_events ());
+    evmgr.install_qt_event_handlers (m_octave_qobj.get_qt_interpreter_events ());
     evmgr.enable ();
 
     int exit_status = 0;
@@ -63,7 +64,8 @@ namespace octave
 
         interp.initialize ();
 
-        if (app_context.start_gui_p ())
+        if (app_context.start_gui_p ()
+            && ! m_octave_qobj.experimental_terminal_widget ())
           {
             input_system& input_sys = interp.get_input_system ();
 
@@ -87,34 +89,23 @@ namespace octave
             exit_status = interp.execute ();
           }
       }
-    catch (const exit_exception& ex)
+    catch (const exit_exception& xe)
       {
-        exit_status = ex.exit_status ();
+        exit_status = xe.exit_status ();
       }
 
-    // Signal that the interpreter is done executing code in the main
-    // REPL, from script files, or command line eval arguments.  By
-    // using a signal here, we give the GUI a chance to process any
-    // pending events, then signal that it is safe to shutdown the
-    // interpreter.  Our notification here allows the GUI to insert the
-    // request to shutdown the interpreter in the event queue after any
-    // other pending signals.  The application context owns the
-    // interpreter and will be responsible for deleting it later, when
-    // the application object destructor is executed.
+    interp.shutdown ();
 
-    emit execution_finished (exit_status);
-  }
-
-  // This function is expected to be executed when the GUI signals that
-  // it is finished processing events and ready for the interpreter to
-  // perform shutdown actions.
-
-  void interpreter_qobject::shutdown (int exit_status)
-  {
-    if (m_interpreter)
-      m_interpreter->shutdown ();
-
-    // Signal that the interpreter has executed shutdown actions.
+    // Signal that the interpreter is done executing code in the
+    // main REPL, from script files, or command line eval arguments.
+    // By using a signal here, we give the GUI a chance to process
+    // any pending events, then signal that it is safe to shutdown
+    // the interpreter.  Our notification here allows the GUI to
+    // insert the request to shutdown the interpreter in the event
+    // queue after any other pending signals.  The application
+    // context owns the interpreter and will be responsible for
+    // deleting it later, when the application object destructor is
+    // executed.
 
     emit shutdown_finished (exit_status);
   }
@@ -139,7 +130,72 @@ namespace octave
     evmgr.post_event (meth);
   }
 
-  qt_interpreter_events * interpreter_qobject::qt_link (void)
+  void interpreter_qobject::interrupt (void)
+  {
+    if (! m_interpreter)
+      return;
+
+    // The following is a direct function call across threads.
+    // We need to ensure that it uses thread-safe functions.
+
+    m_interpreter->interrupt ();
+  }
+
+  void interpreter_qobject::pause (void)
+  {
+    // FIXME: Should we make this action work with the old terminal
+    // widget?
+
+    if (m_octave_qobj.experimental_terminal_widget ())
+      {
+        if (! m_interpreter)
+          return;
+
+        // The following is a direct function call across threads.
+        // We need to ensure that it uses thread-safe functions.
+
+        m_interpreter->pause ();
+      }
+  }
+
+  void interpreter_qobject::stop (void)
+  {
+    // FIXME: Should we make this action work with the old terminal
+    // widget?
+
+    if (m_octave_qobj.experimental_terminal_widget ())
+      {
+        if (! m_interpreter)
+          return;
+
+        // The following is a direct function call across threads.
+        // We need to ensure that it uses thread-safe functions.
+
+        m_interpreter->stop ();
+      }
+  }
+
+  void interpreter_qobject::resume (void)
+  {
+    // FIXME: Should we make this action work with the old terminal
+    // widget?
+
+    if (m_octave_qobj.experimental_terminal_widget ())
+      {
+        // FIXME: This action should only be available when the
+        // interpreter is paused.
+
+        interpreter_event
+          ([=] (interpreter& interp)
+          {
+            // INTERPRETER THREAD
+
+            interp.resume ();
+          });
+      }
+  }
+
+  qt_interpreter_events *interpreter_qobject::qt_link (void)
   {
     return m_octave_qobj.qt_link ();
   }
