@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 1996-2021 The Octave Project Developers
+// Copyright (C) 1996-2022 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -33,8 +33,15 @@
 // FIXME: why don't these classes use iprocstream and oprocstream,
 //        which in turn use the octave_procbuf class?
 
+// Note: Even though these classes are now inside the octave namespace,
+// we can't rename them to iprocstream and oprocstream because we
+// already have classes with those names (see procstream.h).  We need to
+// find a way to resolve this naming issue.
+
+OCTAVE_NAMESPACE_BEGIN
+
 class
-octave_iprocstream : public octave_stdiostream
+octave_iprocstream : public octave::stdiostream
 {
 public:
 
@@ -62,7 +69,7 @@ protected:
 };
 
 class
-octave_oprocstream : public octave_stdiostream
+octave_oprocstream : public octave::stdiostream
 {
 public:
 
@@ -89,4 +96,16 @@ protected:
   ~octave_oprocstream (void);
 };
 
+OCTAVE_NAMESPACE_END
+
+#if defined (OCTAVE_PROVIDE_DEPRECATED_SYMBOLS)
+
+OCTAVE_DEPRECATED (7, "use 'octave::octave_iprocstream' instead")
+typedef octave::octave_iprocstream octave_iprocstream;
+
+OCTAVE_DEPRECATED (7, "use 'octave::octave_oprocstream' instead")
+typedef octave::octave_oprocstream octave_oprocstream;
+
+#endif
+
 #endif
