@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 2013-2020 The Octave Project Developers
+// Copyright (C) 2013-2022 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -52,10 +52,14 @@
 #include "parse.h"
 
 #if defined (HAVE_PORTAUDIO)
+#  include <portaudio.h>
+#endif
 
-#include <portaudio.h>
+OCTAVE_NAMESPACE_BEGIN
 
-PaSampleFormat
+#if defined (HAVE_PORTAUDIO)
+
+static PaSampleFormat
 bits_to_format (int bits)
 {
   if (bits == 8)
@@ -304,7 +308,7 @@ recording using those parameters.
       std::string arg3 = args(2).string_value ();
       std::transform (arg3.begin (), arg3.end (), arg3.begin (), tolower);
       if (arg3 != "driverversion")
-        error ("audiodevinfo: third argument must be \"DriverVersion\"");
+        error (R"(audiodevinfo: third argument must be "DriverVersion")");
 
       if (outin == 0)
         {
@@ -489,18 +493,24 @@ recording using those parameters.
 %!   assert (devinfo.output(i).Name, audiodevinfo (0, devinfo.output(i).ID));
 %! endfor
 %! for i=1:nin
-%!   assert (devinfo.input(i).Name, audiodevinfo (1, devinfo.input(i).ID));
+%!   assert (devinfo.input (i).Name, audiodevinfo (1, devinfo.input (i).ID));
 %! endfor
 
 %!testif HAVE_PORTAUDIO
 %! devinfo = audiodevinfo;
 %! nout = audiodevinfo (0);
 %! nin = audiodevinfo (1);
-%! for i = 1:nout
-%!   assert (devinfo.output(i).ID, audiodevinfo (0, devinfo.output(i).Name));
+%! ## There might be multiple devices with the same name (e.g. on Windows WDM-KS)
+%! ## Check only the first of each unique device name.
+%! [unq_out_name, idx_unique] = unique ({devinfo.output(:).Name});
+%! unq_out_id = [devinfo.output(idx_unique).ID];
+%! for i = 1:numel (unq_out_name)
+%!   assert (audiodevinfo (0, unq_out_name{i}), unq_out_id(i));
 %! endfor
-%! for i = 1:nin
-%!   assert (devinfo.input(i).ID, audiodevinfo (1, devinfo.input(i).Name));
+%! [unq_in_name, idx_unique] = unique ({devinfo.input(:).Name});
+%! unq_in_id = [devinfo.input(idx_unique).ID];
+%! for i = 1:numel (unq_in_name)
+%!   assert (audiodevinfo (1, unq_in_name{i}), unq_in_id(i));
 %! endfor
 */
 
@@ -592,21 +602,21 @@ octave_play_callback (const void *, void *output, unsigned long frames,
   audioplayer *player = static_cast<audioplayer *> (data);
 
   if (! player)
-    error ("audio player callback function called without player");
+    error ("audioplayer callback function called without player");
 
   octave_value_list retval
-    = octave::feval (player->octave_callback_function,
-                     ovl (static_cast<double> (frames)), 1);
+    = feval (player->octave_callback_function,
+             ovl (static_cast<double> (frames)), 1);
 
   if (retval.length () < 2)
-    error ("audio player callback function failed");
+    error ("audioplayer callback function failed");
 
   const Matrix sound = retval(0).matrix_value ();
   int return_status = retval(1).int_value ();
 
   if (frames - sound.rows () != 0 || sound.columns () < 1
       || sound.columns () > 2)
-    error ("audio player callback function failed");
+    error ("audioplayer callback function failed");
 
   // Don't multiply the audio data by scale_factor here.  Although it
   // does move the operation outside of the loops below, it also causes
@@ -653,7 +663,7 @@ octave_play_callback (const void *, void *output, unsigned long frames,
       {
         static double scale_factor = std::pow (2.0, 23) - 1.0;
 
-        static int big_endian = octave::mach_info::words_big_endian ();
+        static int big_endian = mach_info::words_big_endian ();
 
         uint8_t *buffer = static_cast<uint8_t *> (output);
 
@@ -682,7 +692,7 @@ octave_play_callback (const void *, void *output, unsigned long frames,
       break;
 
     default:
-      error ("invalid player bit depth in callback function");
+      error ("invalid bit depth in audioplayer callback function");
     }
 
   return return_status;
@@ -690,13 +700,13 @@ octave_play_callback (const void *, void *output, unsigned long frames,
 
 static int
 portaudio_play_callback (const void *, void *output, unsigned long frames,
-                         const PaStreamCallbackTimeInfo*,
+                         const PaStreamCallbackTimeInfo *,
                          PaStreamCallbackFlags, void *data)
 {
   audioplayer *player = static_cast<audioplayer *> (data);
 
   if (! player)
-    error ("audio player callback function called without player");
+    error ("audioplayer callback function called without player");
 
   // Don't multiply the audio data by scale_factor here.  Although it would
   // move the operation outside of the loops below, it also causes a second
@@ -763,7 +773,7 @@ portaudio_play_callback (const void *, void *output, unsigned long frames,
           {
             static double scale_factor = std::pow (2.0, 23) - 1.0;
 
-            static int big_endian = octave::mach_info::words_big_endian ();
+            static int big_endian = mach_info::words_big_endian ();
 
             uint8_t *buffer = static_cast<uint8_t *> (output);
 
@@ -799,7 +809,7 @@ portaudio_play_callback (const void *, void *output, unsigned long frames,
           break;
 
         default:
-          error ("invalid player bit depth in callback function");
+          error ("invalid bit depth in audioplayer callback function");
         }
     }
   else if (player->get_type () == TYPE_INT8)
@@ -863,12 +873,6 @@ portaudio_play_callback (const void *, void *output, unsigned long frames,
   return paContinue;
 }
 
-static void
-safe_audioplayer_stop (audioplayer *player)
-{
-  player->stop ();
-}
-
 audioplayer::audioplayer (void)
   : octave_callback_function (nullptr),
     id (-1), fs (0), nbits (16), channels (0), sample_number (0),
@@ -1156,9 +1160,7 @@ audioplayer::playblocking (void)
   start = get_sample_number ();
   end = get_end_sample ();
 
-  octave::unwind_protect frame;
-
-  frame.add_fcn (safe_audioplayer_stop, this);
+  unwind_action stop_audioplayer ([=] () { stop (); });
 
   for (unsigned int i = start; i < end; i += buffer_size)
     {
@@ -1349,7 +1351,7 @@ octave_record_callback (const void *input, void *, unsigned long frames,
   audiorecorder *recorder = static_cast<audiorecorder *> (data);
 
   if (! recorder)
-    error ("audio recorder callback function called without recorder");
+    error ("audiorecorder callback function called without recorder");
 
   int channels = recorder->get_channels ();
 
@@ -1367,8 +1369,8 @@ octave_record_callback (const void *input, void *, unsigned long frames,
           float sample_l = input8[i*channels] / scale_factor;
           float sample_r = input8[i*channels + (channels - 1)] / scale_factor;
 
-          sound(i,0) = sample_l;
-          sound(i,1) = sample_r;
+          sound(i, 0) = sample_l;
+          sound(i, 1) = sample_r;
         }
     }
   // FIXME: This is a workaround for a bug in PortAudio affecting 8-Bit
@@ -1387,8 +1389,8 @@ octave_record_callback (const void *input, void *, unsigned long frames,
           float sample_r = (input16[i*channels + (channels - 1)] >> 8)
                            / scale_factor;
 
-          sound(i,0) = sample_l;
-          sound(i,1) = sample_r;
+          sound(i, 0) = sample_l;
+          sound(i, 1) = sample_r;
         }
     }
   else if (recorder->get_sampleFormat () == bits_to_format (16))
@@ -1402,8 +1404,8 @@ octave_record_callback (const void *input, void *, unsigned long frames,
           float sample_l = input16[i*channels] / scale_factor;
           float sample_r = input16[i*channels + (channels - 1)] / scale_factor;
 
-          sound(i,0) = sample_l;
-          sound(i,1) = sample_r;
+          sound(i, 0) = sample_l;
+          sound(i, 1) = sample_r;
         }
     }
   else if (recorder->get_sampleFormat () == bits_to_format (24))
@@ -1434,13 +1436,13 @@ octave_record_callback (const void *input, void *, unsigned long frames,
           if (sample_r32 & 0x00800000)
             sample_r32 |= 0xff000000;
 
-          sound(i,0) = sample_l32 / scale_factor;
-          sound(i,1) = sample_r32 / scale_factor;
+          sound(i, 0) = sample_l32 / scale_factor;
+          sound(i, 1) = sample_r32 / scale_factor;
         }
     }
 
   octave_value_list retval
-    = octave::feval (recorder->octave_callback_function, ovl (sound), 1);
+    = feval (recorder->octave_callback_function, ovl (sound), 1);
 
   return retval(0).int_value ();
 }
@@ -1453,7 +1455,7 @@ portaudio_record_callback (const void *input, void *, unsigned long frames,
   audiorecorder *recorder = static_cast<audiorecorder *> (data);
 
   if (! recorder)
-    error ("audio recorder callback function called without recorder");
+    error ("audiorecorder callback function called without recorder");
 
   int channels = recorder->get_channels ();
 
@@ -1543,12 +1545,6 @@ portaudio_record_callback (const void *input, void *, unsigned long frames,
   return paContinue;
 }
 
-static void
-safe_audiorecorder_stop (audiorecorder *recorder)
-{
-  recorder->stop ();
-}
-
 audiorecorder::audiorecorder (void)
   : octave_callback_function (nullptr),
     id (-1), fs (8000), nbits (8), channels (1), sample_number (0),
@@ -1746,8 +1742,8 @@ audiorecorder::getaudiodata (void)
 
   for (unsigned int i = 0; i < ls; i++)
     {
-      audio(0,i) = left[i];
-      audio(1,i) = right[i];
+      audio(0, i) = left[i];
+      audio(1, i) = right[i];
     }
 
   return audio;
@@ -1836,9 +1832,7 @@ audiorecorder::recordblocking (float seconds)
 
   unsigned int frames = seconds * get_fs ();
 
-  octave::unwind_protect frame;
-
-  frame.add_fcn (safe_audiorecorder_stop, this);
+  unwind_action stop_audiorecorder ([=] () { stop (); });
 
   for (unsigned int i = 0; i < frames; i += buffer_size)
     {
@@ -1972,7 +1966,7 @@ Undocumented internal function.
 #if defined (HAVE_PORTAUDIO)
 
 static audiorecorder *
-get_recorder (octave::interpreter& interp, const octave_value& ov)
+get_recorder (interpreter& interp, const octave_value& ov)
 {
   interp.mlock ();
 
@@ -2419,7 +2413,7 @@ Undocumented internal function.
 #if defined (HAVE_PORTAUDIO)
 
 static audioplayer *
-get_player (octave::interpreter& interp, const octave_value& ov)
+get_player (interpreter& interp, const octave_value& ov)
 {
   interp.mlock ();
 
@@ -2429,7 +2423,7 @@ get_player (octave::interpreter& interp, const octave_value& ov)
 
   audioplayer *pl = dynamic_cast<audioplayer *> (ncrep);
   if (! pl)
-    error ("audiodevinfo.cc get_player: dynamic_cast to audioplayer failed");
+    error ("audiodevinfo.cc (get_player): dynamic_cast to audioplayer failed");
 
   return pl;
 }
@@ -2857,3 +2851,5 @@ Undocumented internal function.
                         "audio playback and recording through PortAudio");
 #endif
 }
+
+OCTAVE_NAMESPACE_END
