@@ -1,4 +1,9 @@
-## Copyright (C) 2000-2019 Kai Habel
+########################################################################
+##
+## Copyright (C) 2000-2022 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -15,6 +20,8 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {@var{q} =} cumtrapz (@var{y})
@@ -42,14 +49,9 @@
 ## @seealso{trapz, cumsum}
 ## @end deftypefn
 
-## Author:      Kai Habel <kai.habel@gmx.de>
-##
-## also: June 2000 Paul Kienzle (fixes,suggestions)
-## 2006-05-12 David Bateman - Modified for NDArrays
-
 function z = cumtrapz (x, y, dim)
 
-  if (nargin < 1) || (nargin > 3)
+  if (nargin < 1)
     print_usage ();
   endif
 
@@ -156,8 +158,7 @@ endfunction
 %!assert (cumtrapz (x2,y,3), reshape ([0,3,8;0,9,20],[1 2 3]))
 
 ## Test input validation
-%!error cumtrapz ()
-%!error cumtrapz (1,2,3,4)
+%!error <Invalid call> cumtrapz ()
 %!error <DIM must be an integer> cumtrapz (1, 2, [1 2])
 %!error <DIM must be an integer> cumtrapz (1, 2, 1.5)
 %!error <DIM must be .* a valid dimension> cumtrapz (1, 2, 0)
