@@ -1,20 +1,27 @@
-## Copyright (C) 2000-2017 Paul Kienzle
+########################################################################
+##
+## Copyright (C) 2000-2022 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
-## Octave is free software; you can redistribute it and/or modify it
+## Octave is free software: you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or (at
-## your option) any later version.
+## the Free Software Foundation, either version 3 of the License, or
+## (at your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-## General Public License for more details.
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
-## <http://www.gnu.org/licenses/>.
+## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {@var{v} =} datevec (@var{date})
@@ -22,19 +29,19 @@
 ## @deftypefnx {} {@var{v} =} datevec (@var{date}, @var{p})
 ## @deftypefnx {} {@var{v} =} datevec (@var{date}, @var{f}, @var{p})
 ## @deftypefnx {} {[@var{y}, @var{m}, @var{d}, @var{h}, @var{mi}, @var{s}] =} datevec (@dots{})
-## Convert a serial date number (see @code{datenum}) or date string (see
-## @code{datestr}) into a date vector.
+## Convert a serial date number (@pxref{XREFdatenum,,@code{datenum}}) or date
+## string (@pxref{XREFdatestr,,@code{datestr}}) into a date vector.
 ##
 ## A date vector is a row vector with six members, representing the year,
 ## month, day, hour, minute, and seconds respectively.
 ##
 ## @var{f} is the format string used to interpret date strings
-## (see @code{datestr}).  If @var{date} is a string, but no format is
-## specified, then a relatively slow search is performed through various
-## formats.  It is always preferable to specify the format string @var{f} if it
-## is known.  Formats which do not specify a particular time component will
-## have the value set to zero.  Formats which do not specify a date will
-## default to January 1st of the current year.
+## (@pxref{XREFdatestr,,@code{datestr}}).  If @var{date} is a string, but no
+## format is specified, then a relatively slow search is performed through
+## various formats.  It is always preferable to specify the format string
+## @var{f} if it is known.  Formats which do not specify a particular time
+## component will have the value set to zero.  Formats which do not specify a
+## date will default to January 1st of the current year.
 ##
 ## @var{p} is the year at the start of the century to which two-digit years
 ## will be referenced.  If not specified, it defaults to the current year minus
@@ -44,11 +51,6 @@
 
 ## Algorithm: Peter Baum (http://vsg.cape.com/~pbaum/date/date0.htm)
 
-## Author: pkienzle <pkienzle@users.sf.net>
-## Modified: bdenney <bill@givebillmoney.com>
-## Created: 10 October 2001 (CVS)
-## Adapted-By: William Poetra Yoga Hadisoeseno <williampoetra@gmail.com>
-
 ## The function __date_str2vec__ is based on datesplit by Bill Denney.
 
 function [y, m, d, h, mi, s] = datevec (date, f = [], p = [])
@@ -92,7 +94,7 @@ function [y, m, d, h, mi, s] = datevec (date, f = [], p = [])
     std_formats{++nfmt} = "mm/dd/yyyy HH:MM";
   endif
 
-  if (nargin < 1 || nargin > 3)
+  if (nargin < 1)
     print_usage ();
   endif
 
@@ -204,6 +206,8 @@ endfunction
 
 function [f, rY, ry, fy, fm, fd, fh, fmi, fs] = __date_vfmt2sfmt__ (f)
 
+  original_f = f;   # Store for error messages.
+
   ## Play safe with percent signs.
   f = strrep (f, "%", "%%");
 
@@ -234,6 +238,30 @@ function [f, rY, ry, fy, fm, fd, fh, fmi, fs] = __date_vfmt2sfmt__ (f)
   f = strrep (f, "MM", "%M");
   f = regexprep (f, '[Ss][Ss]', "%S");
 
+  ## Check for conflicting or repeated fields.
+  ## Only warn, not error, if we may be confused by an original '%'s.
+  if (index (original_f, "%"))
+    err_or_warn = @warning;
+  else
+    err_or_warn = @error;
+  endif
+
+  if (numel (strfind (f, "%Y")) + numel (strfind (f, "%y")) > 1)
+    err_or_warn ("datevec: multiple year specifiers in %s", original_f);
+  elseif (numel (strfind (f, "%m")) + numel (strfind (f, "%b"))
+          + numel (strfind (f, "%B")) > 1)
+    err_or_warn ("datevec: multiple month specifiers in %s", original_f);
+  elseif (numel (strfind (f, "%d")) + numel (strfind (f, "%a"))
+          + numel (strfind (f, "%A")) > 1)
+    err_or_warn ("datevec: multiple day specifiers in %s", original_f);
+  elseif (numel (strfind (f, "%H")) + numel (strfind (f, "%I")) > 1)
+    err_or_warn ("datevec: multiple hour specifiers in %s", original_f);
+  elseif (numel (strfind (f, "%M")) > 1)
+    err_or_warn ("datevec: multiple minute specifiers in %s", original_f);
+  elseif (numel (strfind (f, "%S")) > 1)
+    err_or_warn ("datevec: multiple second specifiers in %s", original_f);
+  endif
+
   rY = rindex (f, "%Y");
   ry = rindex (f, "%y");
 
@@ -250,38 +278,52 @@ endfunction
 
 function [found, y, m, d, h, mi, s] = __date_str2vec__ (ds, p, f, rY, ry, fy, fm, fd, fh, fmi, fs)
 
-  idx = strfind (f, "FFF");
-  if (! isempty (idx))
-    ## Kludge to handle FFF millisecond format since strptime does not.
+  ## Local time zone is irrelevant, and potentially dangerous, when using
+  ## strptime to simply convert a string into a broken down struct tm.
+  ## Set and restore TZ so time is parsed exactly as-is. See bug #36954.
+  TZ_orig = getenv ("TZ");
+  unwind_protect
+    setenv ("TZ", "UTC0");
 
-    ## Find location of FFF in ds.
-    ## Might not match idx because of things like yyyy -> %y.
-    [~, nc] = strptime (ds, f(1:idx-1));
+    idx = strfind (f, "FFF");
+    if (! isempty (idx))
+      ## Kludge to handle FFF millisecond format since strptime does not.
 
-    msec = ds(nc:min (nc+2,end)); # pull 3-digit fractional seconds.
-    msec_idx = find (! isdigit (msec), 1);
+      ## Find location of FFF in ds.
+      ## Might not match idx because of things like yyyy -> %y.
+      [~, nc] = strptime (ds, f(1:idx-1));
 
-    if (! isempty (msec_idx))  # non-digits in msec
-      msec = msec(1:msec_idx-1);
-      msec(end+1:3) = "0";     # pad msec with trailing zeros
-      ds = [ds(1:(nc-1)), msec, ds((nc-1)+msec_idx:end)];  # zero pad ds
-    elseif (numel (msec) < 3)  # less than three digits in msec
-      m_len = numel (msec);
-      msec(end+1:3) = "0";     # pad msec with trailing zeros
-      ds = [ds(1:(nc-1)), msec, ds(nc+m_len:end)];  # zero pad ds as well
-    endif
+      msec = ds(nc:min (nc+2,end)); # pull 3-digit fractional seconds.
+      msec_idx = find (! isdigit (msec), 1);
 
-    ## replace FFF with digits to guarantee match in strptime.
-    f(idx:idx+2) = msec;
+      if (! isempty (msec_idx))  # non-digits in msec
+        msec = msec(1:msec_idx-1);
+        msec(end+1:3) = "0";     # pad msec with trailing zeros
+        ds = [ds(1:(nc-1)), msec, ds((nc-1)+msec_idx:end)];  # zero pad ds
+      elseif (numel (msec) < 3)  # less than three digits in msec
+        m_len = numel (msec);
+        msec(end+1:3) = "0";     # pad msec with trailing zeros
+        ds = [ds(1:(nc-1)), msec, ds(nc+m_len:end)];  # zero pad ds as well
+      endif
 
-    if (nc > 0)
+      ## replace FFF with digits to guarantee match in strptime.
+      f(idx:idx+2) = msec;
+
+      if (nc > 0)
+        [tm, nc] = strptime (ds, f);
+        tm.usec = 1000 * str2double (msec);
+      endif
+
+    else
       [tm, nc] = strptime (ds, f);
-      tm.usec = 1000 * str2double (msec);
     endif
-
-  else
-    [tm, nc] = strptime (ds, f);
-  endif
+  unwind_protect_cleanup
+    if (isempty (TZ_orig))
+      unsetenv ("TZ");
+    else
+      setenv ("TZ", TZ_orig);
+    endif
+  end_unwind_protect
 
   if (nc == columns (ds) + 1)
     found = true;
@@ -362,7 +404,7 @@ endfunction
 %!        [2015,6,1,15,7,12.12])
 
 ## Test structure of return value
-%!test <42334>
+%!test <*42334>
 %! [~, ~, d] = datevec ([1 2; 3 4]);
 %! assert (d, [1 2; 3 4]);
 
@@ -371,11 +413,50 @@ endfunction
 %!test
 %! t = linspace (-2e5, 2e5, 10993);
 %! assert (all (abs (datenum (datevec (t)) - t') < 1e-5));
-%!assert (double (datevec (int64 (datenum ([2014 6 1])))), datevec (datenum ([2014 6 1])))
-%!assert (double (datevec (int64 (datenum ([2014 6 18])))), datevec (datenum ([2014 6 18])))
+%!assert (double (datevec (int64 (datenum ([2014 6 1])))),
+%!        datevec (datenum ([2014 6 1])))
+%!assert (double (datevec (int64 (datenum ([2014 6 18])))),
+%!        datevec (datenum ([2014 6 18])))
+
+## Test parsing of date strings that fall within daylight saving transition
+%!testif ; isunix () <*36954>
+%! zones = { "UTC0"                                 ...
+%!           "EST+5EDT,M3.2.0/2,M11.1.0/2"          ... America/New_York
+%!           "CET-1CEST,M3.5.0/2,M10.5.0/2"         ... Europe/Berlin
+%!           "CLT+4CLST,M8.2.0/0,M5.2.0/0"          ... America/Santiago
+%!           "LHST-10:30LHDT-11,M10.1.0/2,M4.1.0/2" ... Australia/Lord_Howe
+%!           ":America/Caracas"                     ...
+%!         };
+%! TZ_orig = getenv ("TZ");
+%! unwind_protect
+%!   for i = 1:numel (zones)
+%!     setenv ("TZ", zones{i});
+%!     ## These specific times were chosen to test conversion during the loss
+%!     ## of some amount of local time at the start of daylight saving time in
+%!     ## each of the zones listed above.  We test all in each time zone to be
+%!     ## exhaustive, even though each is problematic for only one of the zones.
+%!     assert (datevec ("2017-03-12 02:15:00"), [2017  3 12 2 15 0]);
+%!     assert (datevec ("2017-03-26 02:15:00"), [2017  3 26 2 15 0]);
+%!     assert (datevec ("2017-08-13 00:15:00"), [2017  8 13 0 15 0]);
+%!     assert (datevec ("2017-10-01 02:15:00"), [2017 10  1 2 15 0]);
+%!     ## This tests a one-time loss of 30 minutes in Venezuela's local time
+%!     assert (datevec ("2016-05-01 02:40:00"), [2016  5  1 2 40 0]);
+%!   endfor
+%! unwind_protect_cleanup
+%!   if (isempty (TZ_orig))
+%!     unsetenv ("TZ");
+%!   else
+%!     setenv ("TZ", TZ_orig);
+%!   endif
+%! end_unwind_protect
 
 ## Test input validation
-%!error datevec ()
-%!error datevec (1,2,3,4)
+%!error <Invalid call> datevec ()
 %!error <none of the standard formats match> datevec ("foobar")
 %!error <DATE not parsed correctly with given format> datevec ("foobar", "%d")
+%!error <multiple year specifiers> datevec ("1/2/30", "mm/yy/yy")
+%!error <multiple month specifiers> datevec ("1/2/30", "mm/mm/yy")
+%!error <multiple day specifiers> datevec ("1/2/30", "mm/dd/dd")
+%!error <multiple hour specifiers> datevec ("15:38:21.251", "HH:HH:SS")
+%!error <multiple minute specifiers> datevec ("15:38:21.251", "MM:MM:SS")
+%!error <multiple second specifiers> datevec ("15:38:21.251", "HH:SS:SS")
